@@ -29,29 +29,45 @@ static refill_buffer_cb refill_buffer;
 static uint32_t xsvf_buffer_len, xsvf_pos;
 static unsigned char* xsvf_buffer;
 
-void cpld_jtag_setup(jtag_t* const jtag) {
-	scu_pinmux(SCU_PINMUX_CPLD_TDO, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION4);
-	scu_pinmux(SCU_PINMUX_CPLD_TCK, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION0);
-	scu_pinmux(SCU_PINMUX_CPLD_TMS, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION0);
-	scu_pinmux(SCU_PINMUX_CPLD_TDI, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION0);
-	
-	gpio_input(jtag->gpio->gpio_tdo);
-	gpio_output(jtag->gpio->gpio_tck);
-	gpio_output(jtag->gpio->gpio_tms);
-	gpio_output(jtag->gpio->gpio_tdi);
+void cpld_jtag_take(jtag_t* const jtag) {
+	const jtag_gpio_t* const gpio = jtag->gpio;
+
+	/* Set initial GPIO state to the voltages of the internal or external pull-ups/downs,
+	 * to avoid any glitches.
+	 */
+#ifdef HACKRF_ONE
+	gpio_set(gpio->gpio_pp_tms);
+#endif
+	gpio_set(gpio->gpio_tms);
+	gpio_set(gpio->gpio_tdi);
+	gpio_clear(gpio->gpio_tck);
+
+#ifdef HACKRF_ONE
+	/* Do not drive PortaPack-specific TMS pin initially, just to be cautious. */
+	gpio_input(gpio->gpio_pp_tms);
+	gpio_input(gpio->gpio_pp_tdo);
+#endif
+	gpio_output(gpio->gpio_tms);
+	gpio_output(gpio->gpio_tdi);
+	gpio_output(gpio->gpio_tck);
+	gpio_input(gpio->gpio_tdo);
 }
 
-/* set pins as inputs so we don't interfere with an external JTAG device */
 void cpld_jtag_release(jtag_t* const jtag) {
-	scu_pinmux(SCU_PINMUX_CPLD_TDO, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION4);
-	scu_pinmux(SCU_PINMUX_CPLD_TCK, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION0);
-	scu_pinmux(SCU_PINMUX_CPLD_TMS, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION0);
-	scu_pinmux(SCU_PINMUX_CPLD_TDI, SCU_GPIO_NOPULL | SCU_CONF_FUNCTION0);
-	
-	gpio_input(jtag->gpio->gpio_tdo);
-	gpio_input(jtag->gpio->gpio_tck);
-	gpio_input(jtag->gpio->gpio_tms);
-	gpio_input(jtag->gpio->gpio_tdi);
+	const jtag_gpio_t* const gpio = jtag->gpio;
+
+	/* Make all pins inputs when JTAG interface not active.
+	 * Let the pull-ups/downs do the work.
+	 */
+#ifdef HACKRF_ONE
+	/* Do not drive PortaPack-specific pins, initially, just to be cautious. */
+	gpio_input(gpio->gpio_pp_tms);
+	gpio_input(gpio->gpio_pp_tdo);
+#endif
+	gpio_input(gpio->gpio_tms);
+	gpio_input(gpio->gpio_tdi);
+	gpio_input(gpio->gpio_tck);
+	gpio_input(gpio->gpio_tdo);
 }
 
 /* return 0 if success else return error code see xsvfExecute() */
@@ -62,7 +78,7 @@ int cpld_jtag_program(
         refill_buffer_cb refill
 ) {
 	int error;
-	cpld_jtag_setup(jtag);
+	cpld_jtag_take(jtag);
 	xsvf_buffer = buffer;
 	xsvf_buffer_len = buffer_length;
         refill_buffer = refill;
