@@ -34,7 +34,6 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <fftw3.h>
-#include <math.h>
 #include <inttypes.h>
 
 #define _FILE_OFFSET_BITS 64
@@ -46,8 +45,8 @@ typedef int bool;
 #endif
 
 #ifdef _WIN32
+#define _USE_MATH_DEFINES
 #include <windows.h>
-
 #ifdef _MSC_VER
 
 #ifdef _WIN64
@@ -84,6 +83,7 @@ int gettimeofday(struct timeval *tv, void* ignored) {
 #endif
 
 #include <signal.h>
+#include <math.h>
 
 #define FD_BUFFER_SIZE (8*1024)
 
@@ -93,20 +93,25 @@ int gettimeofday(struct timeval *tv, void* ignored) {
 #define FREQ_MAX_MHZ (7250) /* 7250 MHz */
 
 #define DEFAULT_SAMPLE_RATE_HZ (20000000) /* 20MHz default sample rate */
-#define DEFAULT_BASEBAND_FILTER_BANDWIDTH (15000000) /* 5MHz default */
+#define DEFAULT_BASEBAND_FILTER_BANDWIDTH (15000000) /* 15MHz default */
 
 #define TUNE_STEP (DEFAULT_SAMPLE_RATE_HZ / FREQ_ONE_MHZ)
 #define OFFSET 7500000
 
-#define DEFAULT_SAMPLE_COUNT 0x4000
 #define BLOCKS_PER_TRANSFER 16
+#define THROWAWAY_BLOCKS 2
 
 #if defined _WIN32
-	#define sleep(a) Sleep( (a*1000) )
+	#define m_sleep(a) Sleep( (a) )
+#else
+	#define m_sleep(a) usleep((a*1000))
 #endif
 
+uint32_t num_samples = SAMPLES_PER_BLOCK;
+uint32_t num_sweeps = 0;
 int num_ranges = 0;
 uint16_t frequencies[MAX_SWEEP_RANGES*2];
+int step_count;
 
 static float TimevalDiff(const struct timeval *a, const struct timeval *b) {
    return (a->tv_sec - b->tv_sec) + 1e-6f * (a->tv_usec - b->tv_usec);
@@ -160,11 +165,13 @@ int parse_u32_range(char* s, uint32_t* const value_min, uint32_t* const value_ma
 
 volatile bool do_exit = false;
 
-FILE* fd = NULL;
+FILE* outfile = NULL;
 volatile uint32_t byte_count = 0;
+volatile uint64_t sweep_count = 0;
 
 struct timeval time_start;
 struct timeval t_start;
+struct timeval time_stamp;
 
 bool amp = false;
 uint32_t amp_enable;
@@ -173,26 +180,29 @@ bool antenna = false;
 uint32_t antenna_enable;
 
 bool binary_output = false;
+bool ifft_output = false;
 bool one_shot = false;
+bool finite_mode = false;
 volatile bool sweep_started = false;
 
 int fftSize = 20;
-uint32_t fft_bin_width;
+double fft_bin_width;
 fftwf_complex *fftwIn = NULL;
 fftwf_complex *fftwOut = NULL;
 fftwf_plan fftwPlan = NULL;
+fftwf_complex *ifftwIn = NULL;
+fftwf_complex *ifftwOut = NULL;
+fftwf_plan ifftwPlan = NULL;
+uint32_t ifft_idx = 0;
 float* pwr;
 float* window;
-time_t time_now;
-struct tm *fft_time;
-char time_str[50];
 
 float logPower(fftwf_complex in, float scale)
 {
 	float re = in[0] * scale;
 	float im = in[1] * scale;
 	float magsq = re * re + im * im;
-	return log2f(magsq) * 10.0f / log2(10.0f);
+	return (float) (log2(magsq) * 10.0f / log2(10.0f));
 }
 
 int rx_callback(hackrf_transfer* transfer) {
@@ -201,41 +211,71 @@ int rx_callback(hackrf_transfer* transfer) {
 	uint64_t frequency; /* in Hz */
 	uint64_t band_edge;
 	uint32_t record_length;
-	int i, j;
+	int i, j, ifft_bins;
+	struct tm *fft_time;
+	char time_str[50];
+	struct timeval usb_transfer_time;
 
-	if(NULL == fd) {
+	if(NULL == outfile) {
 		return -1;
 	}
 
+	gettimeofday(&usb_transfer_time, NULL);
 	byte_count += transfer->valid_length;
 	buf = (int8_t*) transfer->buffer;
+	ifft_bins = fftSize * step_count;
 	for(j=0; j<BLOCKS_PER_TRANSFER; j++) {
-		if(do_exit) {
-			return 0;
-		}
 		ubuf = (uint8_t*) buf;
 		if(ubuf[0] == 0x7F && ubuf[1] == 0x7F) {
 			frequency = ((uint64_t)(ubuf[9]) << 56) | ((uint64_t)(ubuf[8]) << 48) | ((uint64_t)(ubuf[7]) << 40)
 					| ((uint64_t)(ubuf[6]) << 32) | ((uint64_t)(ubuf[5]) << 24) | ((uint64_t)(ubuf[4]) << 16)
 					| ((uint64_t)(ubuf[3]) << 8) | ubuf[2];
 		} else {
-			buf += SAMPLES_PER_BLOCK;
+			buf += BYTES_PER_BLOCK;
 			continue;
 		}
-		if(!sweep_started) {
-			if (frequency == (uint64_t)(FREQ_ONE_MHZ*frequencies[0])) {
-				sweep_started = true;
-			} else {
-				buf += SAMPLES_PER_BLOCK;
-				continue;
+		if (frequency == (uint64_t)(FREQ_ONE_MHZ*frequencies[0])) {
+			if(sweep_started) {
+				if(ifft_output) {
+					fftwf_execute(ifftwPlan);
+					for(i=0; i < ifft_bins; i++) {
+						ifftwOut[i][0] *= 1.0f / ifft_bins;
+						ifftwOut[i][1] *= 1.0f / ifft_bins;
+						fwrite(&ifftwOut[i][0], sizeof(float), 1, outfile);
+						fwrite(&ifftwOut[i][1], sizeof(float), 1, outfile);
+					}
+				}
+				sweep_count++;
+				if(one_shot) {
+					do_exit = true;
+				}
+				else if(finite_mode && sweep_count == num_sweeps) {
+					do_exit = true;
+				}
+			}
+			sweep_started = true;
+			time_stamp = usb_transfer_time;
+			time_stamp.tv_usec +=
+					(uint64_t)(num_samples + THROWAWAY_BLOCKS * SAMPLES_PER_BLOCK)
+					* j * FREQ_ONE_MHZ / DEFAULT_SAMPLE_RATE_HZ;
+			if(999999 < time_stamp.tv_usec) {
+				time_stamp.tv_sec += time_stamp.tv_usec / 1000000;
+				time_stamp.tv_usec = time_stamp.tv_usec % 1000000;
 			}
 		}
+		if(do_exit) {
+			return 0;
+		}
+		if(!sweep_started) {
+			buf += BYTES_PER_BLOCK;
+			continue;
+		}
 		if((FREQ_MAX_MHZ * FREQ_ONE_MHZ) < frequency) {
-			buf += SAMPLES_PER_BLOCK;
+			buf += BYTES_PER_BLOCK;
 			continue;
 		}
 		/* copy to fftwIn as floats */
-		buf += SAMPLES_PER_BLOCK - (fftSize * 2);
+		buf += BYTES_PER_BLOCK - (fftSize * 2);
 		for(i=0; i < fftSize; i++) {
 			fftwIn[i][0] = buf[i*2] * window[i] * 1.0f / 128.0f;
 			fftwIn[i][1] = buf[i*2+1] * window[i] * 1.0f / 128.0f;
@@ -249,47 +289,59 @@ int rx_callback(hackrf_transfer* transfer) {
 			record_length = 2 * sizeof(band_edge)
 					+ (fftSize/4) * sizeof(float);
 
-			fwrite(&record_length, sizeof(record_length), 1, stdout);
+			fwrite(&record_length, sizeof(record_length), 1, outfile);
 			band_edge = frequency;
-			fwrite(&band_edge, sizeof(band_edge), 1, stdout);
+			fwrite(&band_edge, sizeof(band_edge), 1, outfile);
 			band_edge = frequency + DEFAULT_SAMPLE_RATE_HZ / 4;
-			fwrite(&band_edge, sizeof(band_edge), 1, stdout);
-			fwrite(&pwr[1+(fftSize*5)/8], sizeof(float), fftSize/4, stdout);
+			fwrite(&band_edge, sizeof(band_edge), 1, outfile);
+			fwrite(&pwr[1+(fftSize*5)/8], sizeof(float), fftSize/4, outfile);
 
-			fwrite(&record_length, sizeof(record_length), 1, stdout);
+			fwrite(&record_length, sizeof(record_length), 1, outfile);
 			band_edge = frequency + DEFAULT_SAMPLE_RATE_HZ / 2;
-			fwrite(&band_edge, sizeof(band_edge), 1, stdout);
+			fwrite(&band_edge, sizeof(band_edge), 1, outfile);
 			band_edge = frequency + (DEFAULT_SAMPLE_RATE_HZ * 3) / 4;
-			fwrite(&band_edge, sizeof(band_edge), 1, stdout);
-			fwrite(&pwr[1+fftSize/8], sizeof(float), fftSize/4, stdout);
+			fwrite(&band_edge, sizeof(band_edge), 1, outfile);
+			fwrite(&pwr[1+fftSize/8], sizeof(float), fftSize/4, outfile);
+		} else if(ifft_output) {
+			ifft_idx = (uint32_t) round((frequency - (uint64_t)(FREQ_ONE_MHZ*frequencies[0]))
+					/ fft_bin_width);
+			ifft_idx = (ifft_idx + ifft_bins/2) % ifft_bins;
+			for(i = 0; (fftSize / 4) > i; i++) {
+				ifftwIn[ifft_idx + i][0] = fftwOut[i + 1 + (fftSize*5)/8][0];
+				ifftwIn[ifft_idx + i][1] = fftwOut[i + 1 + (fftSize*5)/8][1];
+			}
+			ifft_idx += fftSize / 2;
+			ifft_idx %= ifft_bins;
+			for(i = 0; (fftSize / 4) > i; i++) {
+				ifftwIn[ifft_idx + i][0] = fftwOut[i + 1 + (fftSize/8)][0];
+				ifftwIn[ifft_idx + i][1] = fftwOut[i + 1 + (fftSize/8)][1];
+			}
 		} else {
-			time_now = time(NULL);
-			fft_time = localtime(&time_now);
+			time_t time_stamp_seconds = time_stamp.tv_sec;
+			fft_time = localtime(&time_stamp_seconds);
 			strftime(time_str, 50, "%Y-%m-%d, %H:%M:%S", fft_time);
-			printf("%s, %" PRIu64 ", %" PRIu64 ", %.2f, %u",
+			fprintf(outfile, "%s.%06ld, %" PRIu64 ", %" PRIu64 ", %.2f, %u",
 					time_str,
+					(long int)time_stamp.tv_usec,
 					(uint64_t)(frequency),
 					(uint64_t)(frequency+DEFAULT_SAMPLE_RATE_HZ/4),
-					(float)fft_bin_width,
+					fft_bin_width,
 					fftSize);
-			for(i=1+(fftSize*5)/8; (1+(fftSize*7)/8) > i; i++) {
-				printf(", %.2f", pwr[i]);
+			for(i = 0; (fftSize / 4) > i; i++) {
+				fprintf(outfile, ", %.2f", pwr[i + 1 + (fftSize*5)/8]);
 			}
-			printf("\n");
-			printf("%s, %" PRIu64 ", %" PRIu64 ", %.2f, %u",
+			fprintf(outfile, "\n");
+			fprintf(outfile, "%s.%06ld, %" PRIu64 ", %" PRIu64 ", %.2f, %u",
 					time_str,
+					(long int)time_stamp.tv_usec,
 					(uint64_t)(frequency+(DEFAULT_SAMPLE_RATE_HZ/2)),
 					(uint64_t)(frequency+((DEFAULT_SAMPLE_RATE_HZ*3)/4)),
-					(float)fft_bin_width,
+					fft_bin_width,
 					fftSize);
-			for(i=1+fftSize/8; (1+(fftSize*3)/8) > i; i++) {
-				printf(", %.2f", pwr[i]);
+			for(i = 0; (fftSize / 4) > i; i++) {
+				fprintf(outfile, ", %.2f", pwr[i + 1 + (fftSize/8)]);
 			}
-			printf("\n");
-		}
-		if(one_shot && ((uint64_t)(frequency+((DEFAULT_SAMPLE_RATE_HZ*3)/4))
-				>= (uint64_t)(FREQ_ONE_MHZ*frequencies[num_ranges*2-1]))) {
-			do_exit = true;
+			fprintf(outfile, "\n");
 		}
 	}
 	return 0;
@@ -304,10 +356,13 @@ static void usage() {
 	fprintf(stderr, "\t[-p antenna_enable] # Antenna port power, 1=Enable, 0=Disable\n");
 	fprintf(stderr, "\t[-l gain_db] # RX LNA (IF) gain, 0-40dB, 8dB steps\n");
 	fprintf(stderr, "\t[-g gain_db] # RX VGA (baseband) gain, 0-62dB, 2dB steps\n");
-	fprintf(stderr, "\t[-n num_samples] # Number of samples per frequency, 16384-4294967296\n");
+	fprintf(stderr, "\t[-n num_samples] # Number of samples per frequency, 8192-4294967296\n");
 	fprintf(stderr, "\t[-w bin_width] # FFT bin width (frequency resolution) in Hz\n");
 	fprintf(stderr, "\t[-1] # one shot mode\n");
+	fprintf(stderr, "\t[-N num_sweeps] # Number of sweeps to perform\n");
 	fprintf(stderr, "\t[-B] # binary output\n");
+	fprintf(stderr, "\t[-I] # binary inverse FFT output\n");
+	fprintf(stderr, "\t-r filename # output file\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Output fields:\n");
 	fprintf(stderr, "\tdate, time, hz_low, hz_high, hz_bin_width, num_samples, dB, dB, . . .\n");
@@ -334,19 +389,20 @@ void sigint_callback_handler(int signum)  {
 
 int main(int argc, char** argv) {
 	int opt, i, result = 0;
-	const char* path = "/dev/null";
+	const char* path = NULL;
 	const char* serial_number = NULL;
 	int exit_code = EXIT_SUCCESS;
-	struct timeval t_end;
+	struct timeval time_now;
+	struct timeval time_prev;
 	float time_diff;
+	float sweep_rate;
 	unsigned int lna_gain=16, vga_gain=20;
-	uint32_t num_samples = DEFAULT_SAMPLE_COUNT;
-	int step_count;
 	uint32_t freq_min = 0;
 	uint32_t freq_max = 6000;
+	uint32_t requested_fft_bin_width;
 
 
-	while( (opt = getopt(argc, argv, "a:f:p:l:g:d:n:w:1Bh?")) != EOF ) {
+	while( (opt = getopt(argc, argv, "a:f:p:l:g:d:n:N:w:1BIr:h?")) != EOF ) {
 		result = HACKRF_SUCCESS;
 		switch( opt ) 
 		{
@@ -403,9 +459,14 @@ int main(int argc, char** argv) {
 			result = parse_u32(optarg, &num_samples);
 			break;
 
+		case 'N':
+			finite_mode = true;
+			result = parse_u32(optarg, &num_sweeps);
+			break;
+
 		case 'w':
-			result = parse_u32(optarg, &fft_bin_width);
-			fftSize = DEFAULT_SAMPLE_RATE_HZ / fft_bin_width;
+			result = parse_u32(optarg, &requested_fft_bin_width);
+			fftSize = DEFAULT_SAMPLE_RATE_HZ / requested_fft_bin_width;
 			break;
 
 		case '1':
@@ -416,6 +477,14 @@ int main(int argc, char** argv) {
 			binary_output = true;
 			break;
 
+		case 'I':
+			ifft_output = true;
+			break;
+
+		case 'r':
+			path = optarg;
+			break;
+
 		case 'h':
 		case '?':
 			usage();
@@ -441,12 +510,12 @@ int main(int argc, char** argv) {
 		fprintf(stderr, "warning: vga_gain (-g) must be a multiple of 2\n");
 
 	if (num_samples % SAMPLES_PER_BLOCK) {
-		fprintf(stderr, "warning: num_samples (-n) must be a multiple of 16384\n");
+		fprintf(stderr, "warning: num_samples (-n) must be a multiple of 8192\n");
 		return EXIT_FAILURE;
 	}
 
 	if (num_samples < SAMPLES_PER_BLOCK) {
-		fprintf(stderr, "warning: num_samples (-n) must be at least 16384\n");
+		fprintf(stderr, "warning: num_samples (-n) must be at least 8192\n");
 		return EXIT_FAILURE;
 	}
 
@@ -472,15 +541,25 @@ int main(int argc, char** argv) {
 		num_ranges++;
 	}
 
+	if(binary_output && ifft_output) {
+		fprintf(stderr, "argument error: binary output (-B) and IFFT output (-I) are mutually exclusive.\n");
+		return EXIT_FAILURE;
+	}
+
+	if(ifft_output && (1 < num_ranges)) {
+		fprintf(stderr, "argument error: only one frequency range is supported in IFFT output (-I) mode.\n");
+		return EXIT_FAILURE;
+	}
+
 	if(4 > fftSize) {
 		fprintf(stderr,
 				"argument error: FFT bin width (-w) must be no more than one quarter the sample rate\n");
 		return EXIT_FAILURE;
 	}
 
-	if(16368 < fftSize) {
+	if(8184 < fftSize) {
 		fprintf(stderr,
-				"argument error: FFT bin width (-w) too small, resulted in more than 16368 FFT bins\n");
+				"argument error: FFT bin width (-w) too small, resulted in more than 8184 FFT bins\n");
 		return EXIT_FAILURE;
 	}
 
@@ -492,14 +571,14 @@ int main(int argc, char** argv) {
 		fftSize++;
 	}
 
-	fft_bin_width = DEFAULT_SAMPLE_RATE_HZ / fftSize;
+	fft_bin_width = (double)DEFAULT_SAMPLE_RATE_HZ / fftSize;
 	fftwIn = (fftwf_complex*)fftwf_malloc(sizeof(fftwf_complex) * fftSize);
 	fftwOut = (fftwf_complex*)fftwf_malloc(sizeof(fftwf_complex) * fftSize);
 	fftwPlan = fftwf_plan_dft_1d(fftSize, fftwIn, fftwOut, FFTW_FORWARD, FFTW_MEASURE);
 	pwr = (float*)fftwf_malloc(sizeof(float) * fftSize);
 	window = (float*)fftwf_malloc(sizeof(float) * fftSize);
 	for (i = 0; i < fftSize; i++) {
-		window[i] = 0.5f * (1.0f - cos(2 * M_PI * i / (fftSize - 1)));
+		window[i] = (float) (0.5f * (1.0f - cos(2 * M_PI * i / (fftSize - 1))));
 	}
 
 	result = hackrf_init();
@@ -515,20 +594,25 @@ int main(int argc, char** argv) {
 		usage();
 		return EXIT_FAILURE;
 	}
-	
-	fd = fopen(path, "wb");
-	if( fd == NULL ) {
+
+	if((NULL == path) || (strcmp(path, "-") == 0)) {
+		outfile = stdout;
+	} else {
+		outfile = fopen(path, "wb");
+	}
+
+	if(NULL == outfile) {
 		fprintf(stderr, "Failed to open file: %s\n", path);
 		return EXIT_FAILURE;
 	}
-	/* Change fd buffer to have bigger one to store or read data on/to HDD */
-	result = setvbuf(fd , NULL , _IOFBF , FD_BUFFER_SIZE);
+	/* Change outfile buffer to have bigger one to store or read data on/to HDD */
+	result = setvbuf(outfile , NULL , _IOFBF , FD_BUFFER_SIZE);
 	if( result != 0 ) {
 		fprintf(stderr, "setvbuf() failed: %d\n", result);
 		usage();
 		return EXIT_FAILURE;
 	}
-	
+
 #ifdef _MSC_VER
 	SetConsoleCtrlHandler( (PHANDLER_ROUTINE) sighandler, TRUE );
 #else
@@ -561,12 +645,6 @@ int main(int argc, char** argv) {
 
 	result = hackrf_set_vga_gain(device, vga_gain);
 	result |= hackrf_set_lna_gain(device, lna_gain);
-	result |= hackrf_start_rx(device, rx_callback, NULL);
-	if (result != HACKRF_SUCCESS) {
-		fprintf(stderr, "hackrf_start_rx() failed: %s (%d)\n", hackrf_error_name(result), result);
-		usage();
-		return EXIT_FAILURE;
-	}
 
 	/*
 	 * For each range, plan a whole number of tuning steps of a certain
@@ -576,12 +654,18 @@ int main(int argc, char** argv) {
 	for(i = 0; i < num_ranges; i++) {
 		step_count = 1 + (frequencies[2*i+1] - frequencies[2*i] - 1)
 				/ TUNE_STEP;
-		frequencies[2*i+1] = frequencies[2*i] + step_count * TUNE_STEP;
+		frequencies[2*i+1] = (uint16_t) (frequencies[2*i] + step_count * TUNE_STEP);
 		fprintf(stderr, "Sweeping from %u MHz to %u MHz\n",
 				frequencies[2*i], frequencies[2*i+1]);
 	}
 
-	result = hackrf_init_sweep(device, frequencies, num_ranges, num_samples,
+	if(ifft_output) {
+		ifftwIn = (fftwf_complex*)fftwf_malloc(sizeof(fftwf_complex) * fftSize * step_count);
+		ifftwOut = (fftwf_complex*)fftwf_malloc(sizeof(fftwf_complex) * fftSize * step_count);
+		ifftwPlan = fftwf_plan_dft_1d(fftSize * step_count, ifftwIn, ifftwOut, FFTW_BACKWARD, FFTW_MEASURE);
+	}
+
+	result = hackrf_init_sweep(device, frequencies, num_ranges, num_samples * 2,
 			TUNE_STEP * FREQ_ONE_MHZ, OFFSET, INTERLEAVED);
 	if( result != HACKRF_SUCCESS ) {
 		fprintf(stderr, "hackrf_init_sweep() failed: %s (%d)\n",
@@ -589,6 +673,13 @@ int main(int argc, char** argv) {
 		return EXIT_FAILURE;
 	}
 
+	result |= hackrf_start_rx_sweep(device, rx_callback, NULL);
+	if (result != HACKRF_SUCCESS) {
+		fprintf(stderr, "hackrf_start_rx_sweep() failed: %s (%d)\n", hackrf_error_name(result), result);
+		usage();
+		return EXIT_FAILURE;
+	}
+
 	if (amp) {
 		fprintf(stderr, "call hackrf_set_amp_enable(%u)\n", amp_enable);
 		result = hackrf_set_amp_enable(device, (uint8_t)amp_enable);
@@ -612,31 +703,27 @@ int main(int argc, char** argv) {
 	}
 
 	gettimeofday(&t_start, NULL);
-	gettimeofday(&time_start, NULL);
+	time_prev = t_start;
 
 	fprintf(stderr, "Stop with Ctrl-C\n");
 	while((hackrf_is_streaming(device) == HACKRF_TRUE) && (do_exit == false)) {
-		uint32_t byte_count_now;
-		struct timeval time_now;
-		float time_difference, rate;
-		sleep(1);
-		
+		float time_difference;
+		m_sleep(50);
+
 		gettimeofday(&time_now, NULL);
-		
-		byte_count_now = byte_count;
-		byte_count = 0;
-		
-		time_difference = TimevalDiff(&time_now, &time_start);
-		rate = (float)byte_count_now / time_difference;
-		fprintf(stderr, "%4.1f MiB / %5.3f sec = %4.1f MiB/second\n",
-				(byte_count_now / 1e6f), time_difference, (rate / 1e6f) );
+		if (TimevalDiff(&time_now, &time_prev) >= 1.0f) {
+			time_difference = TimevalDiff(&time_now, &t_start);
+			sweep_rate = (float)sweep_count / time_difference;
+			fprintf(stderr, "%" PRIu64 " total sweeps completed, %.2f sweeps/second\n",
+					sweep_count, sweep_rate);
 
-		time_start = time_now;
-
-		if (byte_count_now == 0) {
-			exit_code = EXIT_FAILURE;
-			fprintf(stderr, "\nCouldn't transfer any bytes for one second.\n");
-			break;
+			if (byte_count == 0) {
+				exit_code = EXIT_FAILURE;
+				fprintf(stderr, "\nCouldn't transfer any data for one second.\n");
+				break;
+			}
+			byte_count = 0;
+			time_prev = time_now;
 		}
 	}
 
@@ -648,9 +735,10 @@ int main(int argc, char** argv) {
 			   hackrf_error_name(result), result);
 	}
 
-	gettimeofday(&t_end, NULL);
-	time_diff = TimevalDiff(&t_end, &t_start);
-	fprintf(stderr, "Total time: %5.5f s\n", time_diff);
+	gettimeofday(&time_now, NULL);
+	time_diff = TimevalDiff(&time_now, &t_start);
+	fprintf(stderr, "Total sweeps: %" PRIu64 " in %.5f seconds (%.2f sweeps/second)\n",
+			sweep_count, time_diff, sweep_rate);
 
 	if(device != NULL) {
 		result = hackrf_stop_rx(device);
@@ -673,15 +761,18 @@ int main(int argc, char** argv) {
 		fprintf(stderr, "hackrf_exit() done\n");
 	}
 
-	if(fd != NULL) {
-		fclose(fd);
-		fd = NULL;
-		fprintf(stderr, "fclose(fd) done\n");
+	fflush(outfile);
+	if ( ( outfile != NULL ) && ( outfile != stdout ) ) {
+		fclose(outfile);
+		outfile = NULL;
+		fprintf(stderr, "fclose() done\n");
 	}
 	fftwf_free(fftwIn);
 	fftwf_free(fftwOut);
 	fftwf_free(pwr);
 	fftwf_free(window);
+	fftwf_free(ifftwIn);
+	fftwf_free(ifftwOut);
 	fprintf(stderr, "exit\n");
 	return exit_code;
 }
