@@ -20,6 +20,7 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#define _FILE_OFFSET_BITS 64
 
 #include <hackrf.h>
 
@@ -34,8 +35,6 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#define _FILE_OFFSET_BITS 64
-
 #ifndef bool
 typedef int bool;
 #define true 1
@@ -254,6 +253,22 @@ int parse_u32(char* s, uint32_t* const value) {
 	}
 }
 
+/* Parse frequencies as doubles to take advantage of notation parsing */
+int parse_frequency_i64(char* optarg, char* endptr, int64_t* value) {
+	*value = (int64_t) strtod(optarg, &endptr);
+	if (optarg == endptr) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+	return HACKRF_SUCCESS;
+}
+
+int parse_frequency_u32(char* optarg, char* endptr, uint32_t* value) {
+	*value = (uint32_t) strtod(optarg, &endptr);
+	if (optarg == endptr) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+	return HACKRF_SUCCESS;
+}
 
 static char *stringrev(char *str)
 {
@@ -304,14 +319,14 @@ char* u64toa(uint64_t val, t_u64toa* str)
 
 static volatile bool do_exit = false;
 
-FILE* fd = NULL;
+FILE* file = NULL;
 volatile uint32_t byte_count = 0;
 
 bool signalsource = false;
 uint32_t amplitude = 0;
 
 bool hw_sync = false;
-uint32_t hw_sync_enable;
+uint32_t hw_sync_enable = 0;
 
 bool receive = false;
 bool receive_wav = false;
@@ -365,7 +380,7 @@ int rx_callback(hackrf_transfer* transfer) {
 	size_t bytes_written;
 	unsigned int i;
 
-	if( fd != NULL ) 
+	if( file != NULL )
 	{
 		byte_count += transfer->valid_length;
 		bytes_to_write = transfer->valid_length;
@@ -397,7 +412,7 @@ int rx_callback(hackrf_transfer* transfer) {
 #endif
 		    return 0;
 		} else {
-			bytes_written = fwrite(transfer->buffer, 1, bytes_to_write, fd);
+			bytes_written = fwrite(transfer->buffer, 1, bytes_to_write, file);
 			if ((bytes_written != bytes_to_write)
 				|| (limit_num_samples && (bytes_to_xfer == 0))) {
 				return -1;
@@ -415,7 +430,7 @@ int tx_callback(hackrf_transfer* transfer) {
 	size_t bytes_read;
 	unsigned int i;
 
-	if( fd != NULL )
+	if( file != NULL )
 	{
 		byte_count += transfer->valid_length;
 		bytes_to_read = transfer->valid_length;
@@ -429,15 +444,15 @@ int tx_callback(hackrf_transfer* transfer) {
 			}
 			bytes_to_xfer -= bytes_to_read;
 		}
-		bytes_read = fread(transfer->buffer, 1, bytes_to_read, fd);
+		bytes_read = fread(transfer->buffer, 1, bytes_to_read, file);
 		if (limit_num_samples && (bytes_to_xfer == 0)) {
                                return -1;
 		}
 		if (bytes_read != bytes_to_read) {
                        if (repeat) {
                                fprintf(stderr, "Input file end reached. Rewind to beginning.\n");
-                               rewind(fd);
-                               fread(transfer->buffer + bytes_read, 1, bytes_to_read - bytes_read, fd);
+                               rewind(file);
+                               fread(transfer->buffer + bytes_read, 1, bytes_to_read - bytes_read, file);
 			       return 0;
                        } else {
                                return -1; /* not repeat mode, end of file */
@@ -493,7 +508,7 @@ static void usage() {
 	printf("\t[-l gain_db] # RX LNA (IF) gain, 0-40dB, 8dB steps\n");
 	printf("\t[-g gain_db] # RX VGA (baseband) gain, 0-62dB, 2dB steps\n");
 	printf("\t[-x gain_db] # TX VGA (IF) gain, 0-47dB, 1dB steps\n");
-	printf("\t[-s sample_rate_hz] # Sample rate in Hz (4/8/10/12.5/16/20MHz, default %sMHz).\n",
+	printf("\t[-s sample_rate_hz] # Sample rate in Hz (2-20MHz, default %sMHz).\n",
 		u64toa((DEFAULT_SAMPLE_RATE_HZ/FREQ_ONE_MHZ),&ascii_u64_data1));
 	printf("\t[-n num_samples] # Number of samples to transfer (default is unlimited).\n");
 #ifndef _WIN32
@@ -514,7 +529,7 @@ BOOL WINAPI
 sighandler(int signum)
 {
 	if (CTRL_C_EVENT == signum) {
-		fprintf(stdout, "Caught signal %d\n", signum);
+		fprintf(stderr, "Caught signal %d\n", signum);
 		do_exit = true;
 		return TRUE;
 	}
@@ -523,7 +538,7 @@ sighandler(int signum)
 #else
 void sigint_callback_handler(int signum) 
 {
-	fprintf(stdout, "Caught signal %d\n", signum);
+	fprintf(stderr, "Caught signal %d\n", signum);
 	do_exit = true;
 }
 #endif
@@ -537,8 +552,7 @@ int main(int argc, char** argv) {
 	char date_time[DATE_TIME_MAX_LEN];
 	const char* path = NULL;
 	const char* serial_number = NULL;
-	char* endptr;
-	double f_hz;
+	char* endptr = NULL;
 	int result;
 	time_t rawtime;
 	struct tm * timeinfo;
@@ -584,32 +598,17 @@ int main(int argc, char** argv) {
 			break;
 
 		case 'f':
-			f_hz = strtod(optarg, &endptr);
-			if (optarg == endptr) {
-				result = HACKRF_ERROR_INVALID_PARAM;
-				break;
-			}
-			freq_hz = f_hz;
+			result = parse_frequency_i64(optarg, endptr, &freq_hz);
 			automatic_tuning = true;
 			break;
 
 		case 'i':
-			f_hz = strtod(optarg, &endptr);
-			if (optarg == endptr) {
-				result = HACKRF_ERROR_INVALID_PARAM;
-				break;
-			}
-			if_freq_hz = f_hz;
+			result = parse_frequency_i64(optarg, endptr, &if_freq_hz);
 			if_freq = true;
 			break;
 
 		case 'o':
-			f_hz = strtod(optarg, &endptr);
-			if (optarg == endptr) {
-				result = HACKRF_ERROR_INVALID_PARAM;
-				break;
-			}
-			lo_freq_hz = f_hz;
+			result = parse_frequency_i64(optarg, endptr, &lo_freq_hz);
 			lo_freq = true;
 			break;
 
@@ -641,12 +640,7 @@ int main(int argc, char** argv) {
 			break;
 
 		case 's':
-			f_hz = strtod(optarg, &endptr);
-			if (optarg == endptr) {
-				result = HACKRF_ERROR_INVALID_PARAM;
-				break;
-			}
-			sample_rate_hz = f_hz;
+			result = parse_frequency_u32(optarg, endptr, &sample_rate_hz);
 			sample_rate = true;
 			break;
 
@@ -657,12 +651,7 @@ int main(int argc, char** argv) {
 			break;
 
 		case 'b':
-			f_hz = strtod(optarg, &endptr);
-			if (optarg == endptr) {
-				result = HACKRF_ERROR_INVALID_PARAM;
-				break;
-			}
-			baseband_filter_bw_hz = f_hz;
+			result = parse_frequency_u32(optarg, endptr, &baseband_filter_bw_hz);
 			baseband_filter_bw = true;
 			break;
 
@@ -758,7 +747,7 @@ int main(int argc, char** argv) {
 			freq_hz = if_freq_hz;
 			break;
 		case RF_PATH_FILTER_LOW_PASS:
-			freq_hz = labs(if_freq_hz - lo_freq_hz);
+			freq_hz = (int64_t) labs((long int) (if_freq_hz - lo_freq_hz));
 			break;
 		case RF_PATH_FILTER_HIGH_PASS:
 			freq_hz = if_freq_hz + lo_freq_hz;
@@ -903,24 +892,24 @@ int main(int argc, char** argv) {
 		if( transceiver_mode == TRANSCEIVER_MODE_RX )
 		{
 			if (strcmp(path, "-") == 0) {
-				fd = stdout;
+				file = stdout;
 			} else {
-				fd = fopen(path, "wb");
+				file = fopen(path, "wb");
 			}
 		} else {
 			if (strcmp(path, "-") == 0) {
-				fd = stdin;
+				file = stdin;
 			} else {
-				fd = fopen(path, "rb");
+				file = fopen(path, "rb");
 			}
 		}
 	
-		if( fd == NULL ) {
+		if( file == NULL ) {
 			fprintf(stderr, "Failed to open file: %s\n", path);
 			return EXIT_FAILURE;
 		}
-		/* Change fd buffer to have bigger one to store or read data on/to HDD */
-		result = setvbuf(fd , NULL , _IOFBF , FD_BUFFER_SIZE);
+		/* Change file buffer to have bigger one to store or read data on/to HDD */
+		result = setvbuf(file , NULL , _IOFBF , FD_BUFFER_SIZE);
 		if( result != 0 ) {
 			fprintf(stderr, "setvbuf() failed: %d\n", result);
 			usage();
@@ -931,7 +920,7 @@ int main(int argc, char** argv) {
 	/* Write Wav header */
 	if( receive_wav ) 
 	{
-		fwrite(&wave_file_hdr, 1, sizeof(t_wav_file_hdr), fd);
+		fwrite(&wave_file_hdr, 1, sizeof(t_wav_file_hdr), file);
 	}
 	
 #ifdef _MSC_VER
@@ -963,13 +952,11 @@ int main(int argc, char** argv) {
 		}
 	}
 
-	if(hw_sync) {
-		fprintf(stderr, "call hackrf_set_hw_sync_mode(%d)\n", hw_sync);
-		result = hackrf_set_hw_sync_mode(device, hw_sync_enable ? HW_SYNC_MODE_ON : HW_SYNC_MODE_OFF);
-		if( result != HACKRF_SUCCESS ) {
-			fprintf(stderr, "hackrf_set_hw_sync_mode() failed: %s (%d)\n", hackrf_error_name(result), result);
-			return EXIT_FAILURE;
-		}
+	fprintf(stderr, "call hackrf_set_hw_sync_mode(%d)\n", hw_sync_enable);
+	result = hackrf_set_hw_sync_mode(device, hw_sync_enable ? HW_SYNC_MODE_ON : HW_SYNC_MODE_OFF);
+	if( result != HACKRF_SUCCESS ) {
+		fprintf(stderr, "hackrf_set_hw_sync_mode() failed: %s (%d)\n", hackrf_error_name(result), result);
+		return EXIT_FAILURE;
 	}
 
 	if( transceiver_mode == TRANSCEIVER_MODE_RX ) {
@@ -1058,7 +1045,7 @@ int main(int argc, char** argv) {
 			    	len=_st-stream_head;
 				else
 			    	len=stream_size-stream_head;
-				bytes_written = fwrite(stream_buf+stream_head, 1, len, fd);
+				bytes_written = fwrite(stream_buf+stream_head, 1, len, file);
 				if (len != bytes_written) {
 					fprintf(stderr, "write failed");
 					do_exit=true;
@@ -1077,14 +1064,19 @@ int main(int argc, char** argv) {
 			byte_count_now = byte_count;
 			byte_count = 0;
 			
+			
 			time_difference = TimevalDiff(&time_now, &time_start);
 			rate = (float)byte_count_now / time_difference;
-			fprintf(stderr, "%4.1f MiB / %5.3f sec = %4.1f MiB/second\n",
-					(byte_count_now / 1e6f), time_difference, (rate / 1e6f) );
+			if (byte_count_now == 0 && hw_sync == true && hw_sync_enable != 0) {
+			    fprintf(stderr, "Waiting for sync...\n");
+			} else {
+			    fprintf(stderr, "%4.1f MiB / %5.3f sec = %4.1f MiB/second\n",
+					    (byte_count_now / 1e6f), time_difference, (rate / 1e6f) );
+			}
 
 			time_start = time_now;
 
-			if (byte_count_now == 0) {
+			if (byte_count_now == 0 && (hw_sync == false || hw_sync_enable == 0)) {
 				exit_code = EXIT_FAILURE;
 				fprintf(stderr, "\nCouldn't transfer any bytes for one second.\n");
 				break;
@@ -1134,24 +1126,29 @@ int main(int argc, char** argv) {
 		fprintf(stderr, "hackrf_exit() done\n");
 	}
 
-	if(fd != NULL)
+	if(file != NULL)
 	{
 		if( receive_wav ) 
 		{
 			/* Get size of file */
-			file_pos = ftell(fd);
+			file_pos = ftell(file);
 			/* Update Wav Header */
 			wave_file_hdr.hdr.size = file_pos-8;
 			wave_file_hdr.fmt_chunk.dwSamplesPerSec = sample_rate_hz;
 			wave_file_hdr.fmt_chunk.dwAvgBytesPerSec = wave_file_hdr.fmt_chunk.dwSamplesPerSec*2;
 			wave_file_hdr.data_chunk.chunkSize = file_pos - sizeof(t_wav_file_hdr);
 			/* Overwrite header with updated data */
-			rewind(fd);
-			fwrite(&wave_file_hdr, 1, sizeof(t_wav_file_hdr), fd);
+			rewind(file);
+			fwrite(&wave_file_hdr, 1, sizeof(t_wav_file_hdr), file);
 		}	
-		fclose(fd);
-		fd = NULL;
-		fprintf(stderr, "fclose(fd) done\n");
+		if (file != stdin) {
+			fflush(file);
+		}
+		if ((file != stdout) && (file != stdin)) {
+			fclose(file);
+			file = NULL;
+			fprintf(stderr, "fclose() done\n");
+		}
 	}
 	fprintf(stderr, "exit\n");
 	return exit_code;
